@@ -27,6 +27,13 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: contain.h,v $
+ * Revision 1.57  2003/09/17 05:41:58  csoutheren
+ * Removed recursive includes
+ *
+ * Revision 1.56  2003/09/17 01:18:02  csoutheren
+ * Removed recursive include file system and removed all references
+ * to deprecated coooperative threading support
+ *
  * Revision 1.55  2003/03/31 01:23:56  robertj
  * Added ReadFrom functions for standard container classes such as
  *   PIntArray and PStringList etc
@@ -221,7 +228,6 @@
 
 #include <ptlib/object.h>
 
-
 ///////////////////////////////////////////////////////////////////////////////
 // Abstract container class
 
@@ -773,7 +779,7 @@ class PCollection : public PContainer
 // Fill in all the inline functions
 
 #if P_USE_INLINES
-#include "ptlib/contain.inl"
+#include <ptlib/contain.inl>
 #endif
 
 #endif // _CONTAIN_H
