@@ -27,6 +27,16 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: pprocess.h,v $
+ * Revision 1.61  2003/11/25 08:28:13  rjongbloed
+ * Removed ability to have platform without threads, win16 finally deprecated
+ *
+ * Revision 1.60  2003/09/17 05:41:59  csoutheren
+ * Removed recursive includes
+ *
+ * Revision 1.59  2003/09/17 01:18:02  csoutheren
+ * Removed recursive include file system and removed all references
+ * to deprecated coooperative threading support
+ *
  * Revision 1.58  2002/12/11 22:23:59  robertj
  * Added ability to set user identity temporarily and permanently.
  * Added get and set users group functions.
@@ -204,7 +214,7 @@
  *
  */
 
-
+#ifndef _PPROCESS
 #define _PPROCESS
 
 #ifdef P_USE_PRAGMA
@@ -212,6 +222,7 @@
 #endif
 
 #include <ptlib/mutex.h>
+#include <ptlib/syncpoint.h>
 
 
 /**Create a process.
@@ -708,17 +719,6 @@ class PProcess : public PThread
     /* return the time at which the program was started 
     */
 
-  protected:
-#if !defined(P_PLATFORM_HAS_THREADS)
-
-    virtual void OperatingSystemYield();
-    /* Yield to the platforms operating system. This is an internal function
-       and should never be called by the user. It is provided on platforms
-       that do not provide multiple threads in a single process.
-     */
-
-#endif
-
   private:
     void Construct();
 
@@ -767,20 +767,18 @@ class PProcess : public PThread
     int maxHandles;
     // Maximum number of file handles process can open.
 
-#if !defined(P_PLATFORM_HAS_THREADS)
-
-    PThread * currentThread;
-    // Currently running thread in the process
-
-#endif
-
 
   friend class PThread;
 
 
 // Include platform dependent part of class
-#include <ptlib/pprocess.h>
+#ifdef _WIN32
+#include "msos/ptlib/pprocess.h"
+#else
+#include "unix/ptlib/pprocess.h"
+#endif
 };
 
+#endif
 
 // End Of File ///////////////////////////////////////////////////////////////
