@@ -5,6 +5,21 @@
  *
  * Portable Windows Library
  *
+ * $Log: asnper.cxx,v $
+ * Revision 1.8  2004/01/17 17:43:42  csoutheren
+ * Fixed problem with the upper limit on various constrained types not being correctly enforced
+ *
+ * Revision 1.7  2004/01/17 09:23:43  csoutheren
+ * Fixed problem with the upper limit on constrained unsigned integers not being correctly enforced
+ *
+ * Revision 1.6  2003/12/14 10:21:29  rjongbloed
+ * Fixed bug in length incorrectlty decoded from ASN and (apparently) rare circumstances. Thanks pangxg@hotmail.com.
+ * Cleaned up return values to be BOOL rather than int for some functions.
+ *
+ * Revision 1.5  2003/12/03 03:50:03  csoutheren
+ * Reversed last change as it broke decoding in some circumstances
+ *
+ *
  */
 
 ///////////////////////////////////////////////////////////////////////
@@ -21,7 +36,7 @@ void PPER_Stream::NullEncode(const PASN_Null &)
 
 ///////////////////////////////////////////////////////////////////////
 
-int PASN_ConstrainedObject::ConstrainedLengthDecode(PPER_Stream & strm, unsigned & length)
+BOOL PASN_ConstrainedObject::ConstrainedLengthDecode(PPER_Stream & strm, unsigned & length)
 {
   // The execution order is important in the following. The SingleBitDecode() function
   // must be called if extendable is TRUE, no matter what.
@@ -61,7 +76,7 @@ BOOL PASN_ConstrainedObject::ConstraintEncode(PPER_Stream & strm, unsigned value
 
   strm.SingleBitEncode(needsExtending);
 
-  return needsExtending || constraint != FixedConstraint;
+  return needsExtending;
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -111,7 +126,7 @@ BOOL PASN_Integer::DecodePER(PPER_Stream & strm)
 
     default : // 12.2.6
       unsigned len;
-      if (strm.LengthDecode(0, INT_MAX, len) != 0)
+      if (!strm.LengthDecode(0, INT_MAX, len))
         return FALSE;
 
       len *= 8;
@@ -126,7 +141,7 @@ BOOL PASN_Integer::DecodePER(PPER_Stream & strm)
   }
 
   if ((unsigned)lowerLimit != upperLimit)  // 12.2.2
-    return strm.UnsignedDecode(lowerLimit, upperLimit, value) == 0; // which devolves to 10.5
+    return strm.UnsignedDecode(lowerLimit, upperLimit, value); // which devolves to 10.5
 
   // 12.2.1
   value = lowerLimit;
@@ -188,11 +203,11 @@ BOOL PASN_Enumeration::DecodePER(PPER_Stream & strm)
       unsigned len = 0;
       return strm.SmallUnsignedDecode(len) &&
              len > 0 &&
-             strm.UnsignedDecode(0, len-1, value) == 0;
+             strm.UnsignedDecode(0, len-1, value);
     }
   }
 
-  return strm.UnsignedDecode(0, maxEnumValue, value) == 0;  // 13.2
+  return strm.UnsignedDecode(0, maxEnumValue, value);  // 13.2
 }
 
 
@@ -248,7 +263,7 @@ BOOL PPER_Stream::ObjectIdDecode(PASN_ObjectId & value)
   // X.691 Section 23
 
   unsigned dataLen;
-  if (LengthDecode(0, 255, dataLen) < 0)
+  if (!LengthDecode(0, 255, dataLen))
     return FALSE;
 
   ByteAlign();
@@ -327,7 +342,7 @@ BOOL PASN_BitString::DecodePER(PPER_Stream & strm)
 {
   // X.691 Section 15
 
-  if (ConstrainedLengthDecode(strm, totalBits) < 0)
+  if (!ConstrainedLengthDecode(strm, totalBits))
     return FALSE;
 
   if (!SetSize(totalBits))
@@ -421,7 +436,7 @@ BOOL PASN_OctetString::DecodePER(PPER_Stream & strm)
   // X.691 Section 16
 
   unsigned nBytes;
-  if (ConstrainedLengthDecode(strm, nBytes) < 0)
+  if (!ConstrainedLengthDecode(strm, nBytes))
     return FALSE;
 
   if (!SetSize(nBytes))   // 16.5
@@ -506,7 +521,7 @@ BOOL PASN_ConstrainedString::DecodePER(PPER_Stream & strm)
   // X.691 Section 26
 
   unsigned len;
-  if (ConstrainedLengthDecode(strm, len) < 0)
+  if (!ConstrainedLengthDecode(strm, len))
     return FALSE;
 
   if (len == 0) { // 10.9.3.3
@@ -605,7 +620,7 @@ BOOL PASN_BMPString::DecodePER(PPER_Stream & strm)
   // X.691 Section 26
 
   unsigned len;
-  if (ConstrainedLengthDecode(strm, len) < 0)
+  if (!ConstrainedLengthDecode(strm, len))
     return FALSE;
 
   if ((PINDEX)len > MaximumStringSize)
@@ -690,7 +705,7 @@ BOOL PASN_Choice::DecodePER(PPER_Stream & strm)
       tag += numChoices;
 
       unsigned len;
-      if (strm.LengthDecode(0, INT_MAX, len) != 0)
+      if (!strm.LengthDecode(0, INT_MAX, len))
         return FALSE;
 
       BOOL ok;
@@ -717,7 +732,7 @@ BOOL PASN_Choice::DecodePER(PPER_Stream & strm)
   if (numChoices < 2)
     tag = 0;
   else {
-    if (strm.UnsignedDecode(0, numChoices-1, tag) < 0)
+    if (!strm.UnsignedDecode(0, numChoices-1, tag))
       return FALSE;
   }
 
@@ -831,7 +846,7 @@ BOOL PASN_Sequence::KnownExtensionDecodePER(PPER_Stream & strm, PINDEX fld, PASN
     return TRUE;
 
   unsigned len;
-  if (strm.LengthDecode(0, INT_MAX, len) != 0)
+  if (!strm.LengthDecode(0, INT_MAX, len))
     return FALSE;
 
   PINDEX nextExtensionPosition = strm.GetPosition() + len;
@@ -947,7 +962,7 @@ BOOL PPER_Stream::ArrayDecode(PASN_Array & array)
   array.RemoveAll();
 
   unsigned size;
-  if (array.ConstrainedLengthDecode(*this, size) < 0)
+  if (!array.ConstrainedLengthDecode(*this, size))
     return FALSE;
 
   if (!array.SetSize(size))
@@ -1165,7 +1180,7 @@ BOOL PPER_Stream::SmallUnsignedDecode(unsigned & value)
     return MultiBitDecode(6, value);      // 10.6.1
 
   unsigned len;
-  if (LengthDecode(0, INT_MAX, len) != 0)  // 10.6.2
+  if (!LengthDecode(0, INT_MAX, len))  // 10.6.2
     return FALSE;
 
   ByteAlign();
@@ -1195,24 +1210,25 @@ void PPER_Stream::SmallUnsignedEncode(unsigned value)
 }
 
 
-int PPER_Stream::UnsignedDecode(unsigned lower, unsigned upper, unsigned & value)
+BOOL PPER_Stream::UnsignedDecode(unsigned lower, unsigned upper, unsigned & value)
 {
   // X.691 section 10.5
 
   if (lower == upper) { // 10.5.4
     value = lower;
-    return 0;
+    return TRUE;
   }
 
   if (IsAtEnd())
-    return -1;
+    return FALSE;
 
   unsigned range = (upper - lower) + 1;
   unsigned nBits = CountBits(range);
 
   if (aligned && (range == 0 || range > 255)) { // not 10.5.6 and not 10.5.7.1
     if (nBits > 16) {                           // not 10.5.7.4
-      LengthDecode(1, (nBits+7)/8, nBits);      // 12.2.6
+      if (!LengthDecode(1, (nBits+7)/8, nBits))      // 12.2.6
+        return FALSE;
       nBits *= 8;
     }
     else if (nBits > 8)    // not 10.5.7.2
@@ -1221,10 +1237,15 @@ int PPER_Stream::UnsignedDecode(unsigned lower, unsigned upper, unsigned & value
   }
 
   if (!MultiBitDecode(nBits, value))
-    return -1;
+    return FALSE;
 
   value += lower;
-  return 0;
+
+  // clamp value to upper limit
+  if (value > upper)
+    value = upper;
+
+  return TRUE;
 }
 
 
@@ -1258,17 +1279,23 @@ void PPER_Stream::UnsignedEncode(int value, unsigned lower, unsigned upper)
 }
 
 
-int PPER_Stream::LengthDecode(unsigned lower, unsigned upper, unsigned & len)
+BOOL PPER_Stream::LengthDecode(unsigned lower, unsigned upper, unsigned & len)
 {
   // X.691 section 10.9
 
   if (upper != INT_MAX && !aligned) {
     if (upper - lower > 0xffff)
-      return -1; // 10.9.4.2 unsupported
+      return FALSE; // 10.9.4.2 unsupported
     unsigned base;
     if (!MultiBitDecode(CountBits(upper - lower + 1), base))
-      return -1;
-    return lower + base;   // 10.9.4.1
+      return FALSE;
+    len = lower + base;   // 10.9.4.1
+
+    // clamp value to upper limit
+    if (len > upper)
+      len = upper;
+
+    return TRUE;
   }
 
   if (upper < 65536)  // 10.9.3.3
@@ -1277,15 +1304,23 @@ int PPER_Stream::LengthDecode(unsigned lower, unsigned upper, unsigned & len)
   // 10.9.3.5
   ByteAlign();
   if (IsAtEnd())
-    return -1;
+    return FALSE;
 
-  if (SingleBitDecode() == 0)
-    return MultiBitDecode(7, len) ? 0 : -1;   // 10.9.3.6
+  if (SingleBitDecode() == 0) {
+    if (!MultiBitDecode(7, len))   // 10.9.3.6
+      return FALSE;                // 10.9.3.8 unsupported
+  }
 
-  if (SingleBitDecode() == 0)
-    return MultiBitDecode(14, len) ? 0 : -1;    // 10.9.3.7
+  else if (SingleBitDecode() == 0) {
+    if (!MultiBitDecode(14, len))    // 10.9.3.7
+      return FALSE;                  // 10.9.3.8 unsupported
+  }
 
-  return -1;  // 10.9.3.8 unsupported
+  // clamp value to upper limit
+  if (len > upper)
+    len = upper;
+
+  return TRUE;  
 }
 
 
@@ -1338,18 +1373,3 @@ void PPER_Stream::AnyTypeEncode(const PASN_Object * value)
 }
 
 ///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////
