@@ -27,6 +27,39 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: contain.cxx,v $
+ * Revision 1.143  2004/02/15 03:04:52  rjongbloed
+ * Fixed problem with PSortedList nil variable and assignment between instances,
+ *   pointed out by Ben Lear.
+ *
+ * Revision 1.142  2004/02/11 05:09:14  csoutheren
+ * Fixed problems with regex libraries on Solaris, and with host OS numbering
+ * being a quoted string rather than a number. Thanks to Chad Attermann
+ * Fixed problems SSL detection problems thanks to Michal Zygmuntowicz
+ *
+ * Revision 1.141  2004/02/08 11:13:20  rjongbloed
+ * Fixed crash in heavily loaded multi-threaded systems using simultaneous sorted
+ *   lists, Thanks Federico Pinna, Fabrizio Ammollo and the gang at Reitek S.p.A.
+ *
+ * Revision 1.140  2004/01/17 17:28:25  csoutheren
+ * Changed PString::Empty to be inline in header file
+ *
+ * Revision 1.139  2004/01/16 13:24:38  csoutheren
+ * Changed PString::Empty to be thread-safe
+ * Fixed PContainer::SetMinSize and PAbstractArray::SetSize, thanks to 123@call2ua.com
+ * Fixed PString::FindLast, thanks to Andreas Sikkema
+ *
+ * Revision 1.138  2003/12/14 01:12:00  csoutheren
+ * Added return value to PRegularExpression::operator = again (Doh!)
+ *
+ * Revision 1.137  2003/12/13 23:08:46  csoutheren
+ * Changed PRegularExpression to allow a copy constructor and operator =
+ *
+ * Revision 1.136  2003/12/04 13:12:41  csoutheren
+ * Fixed error in PRegularExpression that caused double delete when incorrect regular expression used
+ *
+ * Revision 1.135  2003/09/17 09:02:13  csoutheren
+ * Removed memory leak detection code
+ *
  * Revision 1.134  2003/07/28 18:44:01  dsandras
  * Make use of the libc regex on Linux.
  *
@@ -497,13 +530,13 @@
  * Fixed header comment for RCS.
  */
 
-#if P_MACOS
-#include <sys/types.h>
-extern "C"  // curse apple
-{
-#include <regex.h>
-};
-#endif
+//#if P_MACOS
+//#include <sys/types.h>
+//extern "C"  // curse apple
+//{
+//#include <regex.h>
+//};
+//#endif
 
 #include <ptlib.h>
 
@@ -513,13 +546,13 @@ extern "C"  // curse apple
 extern "C" int vsprintf(char *, const char *, va_list);
 #endif
 
-#if !P_MACOS
-#ifdef  P_LINUX
+#if P_REGEX
+extern "C" {
 #include <regex.h>
+};
 #else
 #include "regex/regex.h"
 #endif
-#endif
 
 #if !P_USE_INLINES
 #include "ptlib/contain.inl"
@@ -594,6 +627,8 @@ void PContainer::Destruct()
 BOOL PContainer::SetMinSize(PINDEX minSize)
 {
   PASSERTINDEX(minSize);
+  if (minSize < 0)
+    minSize = 0;
   if (minSize < GetSize())
     minSize = GetSize();
   return SetSize(minSize);
@@ -751,6 +786,9 @@ PObject::Comparison PAbstractArray::Compare(const PObject & obj) const
 
 BOOL PAbstractArray::SetSize(PINDEX newSize)
 {
+  if (newSize < 0)
+    newSize = 0;
+
   PINDEX newsizebytes = elementSize*newSize;
   PINDEX oldsizebytes = elementSize*GetSize();
 
@@ -1036,19 +1074,6 @@ BOOL PBitArray::Concatenate(const PBitArray & array)
 
 ///////////////////////////////////////////////////////////////////////////////
 
-const PString & PString::Empty()
-{
-#if PMEMORY_CHECK
-  BOOL ignoreAllocations = PMemoryHeap::SetIgnoreAllocations(TRUE);
-#endif
-  static PString emptyString((const char *)NULL);
-#if PMEMORY_CHECK
-  PMemoryHeap::SetIgnoreAllocations(ignoreAllocations);
-#endif
-  return emptyString;
-}
-
-
 PString::PString(const char * cstr)
   : PCharArray(cstr != NULL ? strlen(cstr)+1 : 1)
 {
@@ -1831,7 +1856,7 @@ PINDEX PString::FindLast(const char * cstr, PINDEX offset) const
   }
 
   // search for a matching substring
-  while (offset > 0) {
+  while (offset >= 0) {
     if (strSum == cstrSum && InternalCompare(offset, clen, cstr) == EqualTo)
       return offset;
     strSum += toupper(theArray[--offset]);
@@ -2710,11 +2735,11 @@ PINDEX PSortedStringList::GetNextStringsIndex(const PString & str) const
 {
   PINDEX len = str.GetLength();
 
-  info->lastIndex = InternalStringSelect(str, len, info->root, info->lastElement);
+  info->lastIndex = InternalStringSelect(str, len, info->root);
 
   if (info->lastIndex != 0) {
     Element * prev;
-    while ((prev = info->lastElement->Predecessor()) != &Element::nil &&
+    while ((prev = Predecessor(info->lastElement)) != &info->nil &&
                     ((PString *)prev->data)->NumCompare(str, len) >= EqualTo) {
       info->lastElement = prev;
       info->lastIndex--;
@@ -2727,24 +2752,23 @@ PINDEX PSortedStringList::GetNextStringsIndex(const PString & str) const
 
 PINDEX PSortedStringList::InternalStringSelect(const char * str,
                                                PINDEX len,
-                                               Element * thisElement,
-                                               Element * & lastElement)
+                                               Element * thisElement) const
 {
-  if (thisElement == &Element::nil)
+  if (thisElement == &info->nil)
     return 0;
 
   switch (((PString *)thisElement->data)->NumCompare(str, len)) {
     case PObject::LessThan :
     {
-      PINDEX index = InternalStringSelect(str, len, thisElement->right, lastElement);
+      PINDEX index = InternalStringSelect(str, len, thisElement->right);
       return thisElement->left->subTreeSize + index + 1;
     }
 
     case PObject::GreaterThan :
-      return InternalStringSelect(str, len, thisElement->left, lastElement);
+      return InternalStringSelect(str, len, thisElement->left);
 
     default :
-      lastElement = thisElement;
+      info->lastElement = thisElement;
       return thisElement->left->subTreeSize;
   }
 }
@@ -2880,8 +2904,9 @@ void PStringToString::ReadFrom(istream & strm)
 
 PRegularExpression::PRegularExpression()
 {
-  lastError = NotCompiled;
-  expression = NULL;
+  lastError   = NotCompiled;
+  expression  = NULL;
+  flagsSaved  = IgnoreCase;
 }
 
 
@@ -2898,6 +2923,22 @@ PRegularExpression::PRegularExpression(const char * pattern, int flags)
   Compile(pattern, flags);
 }
 
+PRegularExpression::PRegularExpression(const PRegularExpression & from)
+{
+  expression   = NULL;
+  patternSaved = from.patternSaved;
+  flagsSaved   = from.flagsSaved;
+  Compile(patternSaved, flagsSaved);
+}
+
+PRegularExpression & PRegularExpression::operator =(const PRegularExpression & from)
+{
+  expression   = NULL;
+  patternSaved = from.patternSaved;
+  flagsSaved   = from.flagsSaved;
+  Compile(patternSaved, flagsSaved);
+  return *this;
+}
 
 PRegularExpression::~PRegularExpression()
 {
@@ -2930,15 +2971,20 @@ BOOL PRegularExpression::Compile(const PString & pattern, int flags)
 
 BOOL PRegularExpression::Compile(const char * pattern, int flags)
 {
+  patternSaved = pattern;
+  flagsSaved   = flags;
+
   if (expression != NULL) {
     regfree(expression);
     delete expression;
+    expression = NULL;
   }
   if (pattern == NULL || *pattern == '\0')
-    return BadPattern;
-
-  expression = new regex_t;
-  lastError = regcomp(expression, pattern, flags);
+    lastError = BadPattern;
+  else {
+    expression = new regex_t;
+    lastError = regcomp(expression, pattern, flags);
+  }
   return lastError == NoError;
 }
 
