@@ -27,6 +27,19 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: osutils.cxx,v $
+ * Revision 1.209  2003/11/13 21:42:32  csoutheren
+ * Fixed problem with thread name display under Windows thanks to Ted Szoczei
+ *
+ * Revision 1.208  2003/11/08 01:42:19  rjongbloed
+ * Added thread names to DevStudio display, thanks Ted Szoczei
+ *
+ * Revision 1.207  2003/09/17 09:02:14  csoutheren
+ * Removed memory leak detection code
+ *
+ * Revision 1.206  2003/09/17 01:18:03  csoutheren
+ * Removed recursive include file system and removed all references
+ * to deprecated coooperative threading support
+ *
  * Revision 1.205  2003/07/24 22:01:42  dereksmithies
  * Add fixes from Peter Nixon  for fixing install problems. Thanks.
  *
@@ -770,14 +783,7 @@ void PTrace::Initialise(unsigned level, const char * filename, unsigned options)
   PProcess & process = PProcess::Current();
 #endif
 
-#if PMEMORY_CHECK
-  int ignoreAllocations = -1;
-#endif
-
   if (filename != NULL) {
-#if PMEMORY_CHECK
-    ignoreAllocations = PMemoryHeap::SetIgnoreAllocations(TRUE) ? 1 : 0;
-#endif
     PTextFile * traceOutput;
     if (options & AppendToFile) {
       traceOutput = new PTextFile(filename, PFile::ReadWrite);
@@ -802,11 +808,6 @@ void PTrace::Initialise(unsigned level, const char * filename, unsigned options)
          << " on " << process.GetOSClass() << ' ' << process.GetOSName()
          << " (" << process.GetOSVersion() << '-' << process.GetOSHardware()
          << ") at " << PTime().AsString("yyyy/M/d h:mm:ss.uuu"));
-
-#if PMEMORY_CHECK
-  if (ignoreAllocations >= 0)
-    PMemoryHeap::SetIgnoreAllocations(ignoreAllocations != 0);
-#endif
 }
 
 
@@ -1094,10 +1095,8 @@ void PTimer::StartRunning(BOOL once)
   oneshot = once;
   state = (*this) != 0 ? Starting : Stopped;
 
-#if defined(P_PLATFORM_HAS_THREADS)
   if (IsRunning())
     PProcess::Current().SignalTimerChange();
-#endif
 
   // This must have been set by the caller
   timerList->processingMutex.Signal();
@@ -1744,11 +1743,6 @@ int PProcess::p_argc;
 char ** PProcess::p_argv;
 char ** PProcess::p_envp;
 
-#ifndef P_PLATFORM_HAS_THREADS
-static BOOL PProcessTerminating = FALSE;
-#endif
-
-
 PProcess::PProcess(const char * manuf, const char * name,
                            WORD major, WORD minor, CodeStatus stat, WORD build)
   : manufacturer(manuf), productName(name)
@@ -1765,13 +1759,7 @@ PProcess::PProcess(const char * manuf, const char * name,
   // cannot assure destruction at the right time we simply allocate it and
   // NEVER destroy it! This is OK as the only reason for its destruction is
   // the program is exiting and then who cares?
-#if PMEMORY_CHECK
-  BOOL ignoreAllocations = PMemoryHeap::SetIgnoreAllocations(TRUE);
-#endif
   PTraceMutex = new PMutex;
-#if PMEMORY_CHECK
-  PMemoryHeap::SetIgnoreAllocations(ignoreAllocations);
-#endif
 
 #ifndef P_RTEMS
   if (p_argv != 0 && p_argc > 0) {
@@ -1807,9 +1795,6 @@ int PProcess::_main(void *)
 
 void PProcess::PreInitialise(int c, char ** v, char ** e)
 {
-#if PMEMORY_CHECK
-  PMemoryHeap::SetIgnoreAllocations(FALSE);
-#endif
   p_argc = c;
   p_argv = v;
   p_envp = e;
@@ -1846,17 +1831,9 @@ void PProcess::Terminate()
 {
 #ifdef _WINDLL
   FatalExit(terminationValue);
-#else
-#ifndef P_PLATFORM_HAS_THREADS
-  // Can only terminate process from the processes thread
-  if (currentThread == this)
-    exit(terminationValue);
-  else
-    PProcessTerminating = TRUE;
 #else
   exit(terminationValue);
 #endif
-#endif
 }
 
 
@@ -1904,16 +1881,45 @@ PString PThread::GetThreadName() const
   return threadName; 
 }
 
- 
+#if defined(_MSC_VER)
+
+typedef struct tagTHREADNAME_INFO
+{
+  DWORD dwType ;                       // must be 0x1000
+  LPCSTR szName ;                      // pointer to name (in user addr space)
+  DWORD dwThreadID ;                   // thread ID (-1=caller thread, but seems to set more than one thread's name)
+  DWORD dwFlags ;                      // reserved for future use, must be zero
+} THREADNAME_INFO ;
+
+
+void SetWinDebugThreadName (THREADNAME_INFO * info)
+{
+  __try
+  {
+    RaiseException (0x406D1388, 0, sizeof(THREADNAME_INFO)/sizeof(DWORD), (DWORD *) info) ;
+  }                              // if not running under debugger exception comes back
+  __except(EXCEPTION_CONTINUE_EXECUTION)
+  {                              // just keep on truckin'
+  }
+}
+#endif
+
+
 void PThread::SetThreadName(const PString & name)
 {
   if (name.IsEmpty())
     threadName = psprintf("%s:%08x", GetClass(), (INT)this);
   else
     threadName = psprintf(name, (INT)this);
+
+#if defined(_MSC_VER)
+  if (threadId) {       // make thread name known to debugger
+    THREADNAME_INFO Info = { 0x1000, (const char *) threadName, threadId, 0 } ;
+    SetWinDebugThreadName (&Info) ;
+  }
+#endif
 }
-
-
+ 
 PThread * PThread::Create(const PNotifier & notifier,
                           INT parameter,
                           AutoDeleteFlag deletion,
@@ -1956,443 +1962,6 @@ void PSimpleThread::Main()
   callback(*this, parameter);
 }
 
-
-#ifndef P_PLATFORM_HAS_THREADS
-
-void PThread::InitialiseProcessThread()
-{
-  basePriority = NormalPriority;  // User settable priority
-  dynamicPriority = 0;            // Only thing running
-  suspendCount = 0;               // Not suspended (would not be a good idea)
-  ClearBlock();                   // No I/O blocking function
-  status = Running;               // Thread is already running
-  stackBase = NULL;
-  link = this;
-  ((PProcess*)this)->currentThread = this;
-  traceBlockIndentLevel = 0;
-}
-
-
-PThread::PThread(PINDEX stackSize,
-                 AutoDeleteFlag deletion,
-                 Priority priorityLevel,
-                 const PString & name)
-  : threadName(name)
-{
-  autoDelete = deletion == AutoDeleteThread;
-  basePriority = priorityLevel;   // Threads user settable priority level
-  dynamicPriority = 0;            // Run immediately
-  suspendCount = 1;
-  traceBlockIndentLevel = 0;
-
-  AllocateStack(stackSize);
-  PAssert(stackBase != NULL, "Insufficient near heap for thread");
-
-  status = Terminated; // Set to this so Restart() works
-  Restart();
-}
-
-
-void PThread::Restart()
-{
-  if (status != Terminated) // Is already running
-    return;
-
-  ClearBlock();             // No I/O blocking function
-
-  PThread * current = Current();
-  link = current->link;
-  current->link = this;
-
-  status = Starting;
-}
-
-
-void PThread::Terminate()
-{
-  if (link == this || status == Terminated)
-    return;   // Is only thread or already terminated
-
-  if (status == Running) {
-    status = Terminating;
-    Yield(); // Never returns from here
-  }
-
-  PThread * prev = PThread::Current();
-  while (prev->link != this)
-    prev = prev->link;
-  prev->link = link;   // Unlink it from the list
-  status = Terminated;
-}
-
-
-void PThread::WaitForTermination() const
-{
-  while (!IsTerminated())
-    Yield();
-}
-
-
-BOOL PThread::WaitForTermination(const PTimeInterval & maxWait) const
-{
-  PTimer timeout = maxWait;
-  while (!IsTerminated()) {
-    if (timeout == 0)
-      return FALSE;
-    Yield();
-  }
-
-  return TRUE;
-}
-
-
-void PThread::SetAutoDelete(AutoDeleteFlag deletion)
-{
-  PAssert(deletion != AutoDeleteThread || this != &PProcess::Current(), PLogicError);
-  autoDelete = deletion == AutoDeleteThread;
-}
-
-
-void PThread::Suspend(BOOL susp)
-{
-  // Suspend/Resume the thread
-  if (susp)
-    suspendCount++;
-  else
-    suspendCount--;
-
-  switch (status) {
-    case Running : // Suspending itself, yield to next thread
-      if (IsSuspended()) {
-        status = Suspended;
-        Yield();
-      }
-      break;
-
-    case Waiting :
-      if (IsSuspended())
-        status = Suspended;
-      break;
-
-    case BlockedIO :
-      if (IsSuspended())
-        status = SuspendedBlockIO;
-      break;
-
-    case BlockedSem :
-      if (IsSuspended())
-        status = SuspendedBlockSem;
-      break;
-
-    case Suspended :
-      if (!IsSuspended())
-        status = Waiting;
-      break;
-
-    case SuspendedBlockIO :
-      if (!IsSuspended())
-        status = BlockedIO;
-      break;
-
-    case SuspendedBlockSem :
-      if (!IsSuspended())
-        status = BlockedSem;
-      break;
-
-    default :
-      break;
-  }
-}
-
-
-void PThread::Sleep(const PTimeInterval & time)
-{
-  sleepTimer = time;
-  if (time == PMaxTimeInterval)
-    sleepTimer.Pause();
-
-  switch (status) {
-    case Running : // Suspending itself, yield to next thread
-      status = Sleeping;
-      Yield();
-      break;
-
-    case Waiting :
-    case Suspended :
-      status = Sleeping;
-      break;
-
-    default :
-      break;
-  }
-}
-
-
-void PThread::BeginThread()
-{
-  if (IsSuspended()) { // Begins suspended
-    status = Suspended;
-    Yield();
-  }
-  else
-    status = Running;
-
-  Main();
-
-  status = Terminating;
-  Yield(); // Never returns from here
-}
-
-
-void PThread::Yield()
-{
-  PThread * current = PProcessInstance->currentThread;
-  if (current == PProcessInstance) {
-    PProcessInstance->GetTimerList()->Process();
-    if (current->link == current)
-      return;
-  }
-  else {
-    char stackUsed;
-    if (&stackUsed < current->stackBase) {
-      char * buf = (char *)malloc(1000);
-      sprintf(buf, "Stack overflow!\n"
-                   "\n"
-                   "Thread: 0x%08x - %s\n"
-                   "Stack top  : 0x%08x\n"
-                   "Stack base : 0x%08x\n"
-                   "Stack frame: 0x%08x\n"
-                   "\n",
-              (int)current, current->GetClass(),
-              (int)current->stackTop,
-              (int)current->stackBase,
-              (int)&stackUsed);
-      PAssertAlways(buf);
-      PError << "Aborting." << endl;
-      _exit(1);
-    }
-  }
-
-  if (current->status == Running && current->basePriority == HighestPriority)
-    return;
-
-  do {
-    if (current->status == Running)
-      current->status = Waiting;
-
-    static const int dynamicLevel[NumPriorities] = { -1, 3, 1, 0, 0 };
-    current->dynamicPriority = dynamicLevel[current->basePriority];
-
-    PThread * next = NULL; // Next thread to be scheduled
-    PThread * prev = current; // Need the thread in the previous link
-    PThread * start = current;  // Need thread in list that is the "start"
-    PThread * thread = current->link;
-    BOOL pass = 0;
-    BOOL canUseLowest = TRUE;
-
-    for (;;) {
-      if (PProcessTerminating) {
-        next = PProcessInstance;
-        next->status = Running;
-        break;
-      }
-
-      switch (thread->status) {
-        case Waiting :
-          if (thread->dynamicPriority == 0) {
-            next = thread;
-            next->status = Running;
-          }
-          else if (thread->dynamicPriority > 0) {
-            thread->dynamicPriority--;
-            canUseLowest = FALSE;
-          }
-          else if (pass > 1 && canUseLowest)
-            thread->dynamicPriority++;
-          break;
-
-        case Sleeping :
-          if (thread->sleepTimer == 0) {
-            if (thread->IsSuspended())
-              thread->status = Suspended;
-            else {
-              thread->status = Running;
-              next = thread;
-            }
-          }
-          break;
-
-        case BlockedIO :
-          if (thread->IsNoLongerBlocked()) {
-            if (PProcessTerminating)
-              next = PProcessInstance;
-            else {
-              thread->ClearBlock();
-              next = thread;
-            }
-            next->status = Running;
-          }
-          break;
-
-        case BlockedSem :
-        case SuspendedBlockSem :
-          if (thread->blockingSemaphore->timeout == 0) {
-            thread->blockingSemaphore->PSemaphore::Signal();
-            thread->blockingSemaphore->timeout = 0;
-            if (thread->status == Waiting) {
-              next = thread;
-              next->status = Running;
-            }
-          }
-          break;
-
-        case Starting :
-          if (!thread->IsSuspended())
-            next = thread;
-          break;
-
-        case Terminating :
-          if (thread == current)         // Cannot self terminate
-            next = PProcessInstance;     // So switch to process thread first
-          else {
-            prev->link = thread->link;   // Unlink it from the list
-            if (thread == start)         // If unlinking the "start" thread
-              start = prev;              //    then we better make it still in list
-            thread->status = Terminated; // Flag thread as terminated
-            if (thread->autoDelete)
-              delete thread;             // Destroy if auto-delete
-            thread = prev;
-          }
-          break;
-
-        default :
-          break;
-      }
-
-      if (next != NULL)  // Have a thread to run
-        break;
-
-      // Need to have previous thread so can unlink a terminating thread
-      prev = thread;
-      thread = thread->link;
-      if (thread == start) {
-        pass++;
-        if (pass > 3) // Everything is blocked
-          PProcessInstance->OperatingSystemYield();
-      }
-    }
-
-    PProcessInstance->currentThread = next;
-
-    next->SwitchContext(current);
-
-    // Could get here with a self terminating thread, so go around again if all
-    // we did was switch stacks, and do not actually have a running thread.
-  } while (PProcessInstance->currentThread->status != Running);
-
-  if (PProcessTerminating) {
-    PProcessTerminating = FALSE;
-    if (PProcessInstance->IsDescendant(PServiceProcess::Class()))
-      ((PServiceProcess *)PProcessInstance)->OnStop();
-    exit(PProcessInstance->terminationValue);
-  }
-}
-
-
-#endif
-
-
-///////////////////////////////////////////////////////////////////////////////
-// PSemaphore
-
-#ifndef P_PLATFORM_HAS_THREADS
-
-PSemaphore::PSemaphore(unsigned initial, unsigned maxCount)
-{
-  PAssert(maxCount > 0, "Invalid semaphore maximum.");
-  if (initial > maxCount)
-    initial = maxCount;
-  currentCount = initial;
-  maximumCount = maxCount;
-}
-
-
-PSemaphore::~PSemaphore()
-{
-  PAssert(blockedThreads.IsEmpty(),
-                        "Semaphore destroyed while still has blocked threads");
-}
-
-
-void PSemaphore::Wait()
-{
-  Wait(PMaxTimeInterval);
-}
-
-
-BOOL PSemaphore::Wait(const PTimeInterval & time)
-{
-  if (currentCount > 0)
-    currentCount--;
-  else {
-    PThread * thread = PThread::Current();
-    blockedThreads.Enqueue(thread);
-    thread->blockingSemaphore = this;
-    thread->status = PThread::BlockedSem;
-    timeout = time;
-    if (time == PMaxTimeInterval)
-      timeout.Pause();
-    PThread::Yield();
-    if (timeout == 0)
-      return FALSE;
-  }
-  return TRUE;
-}
-
-
-void PSemaphore::Signal()
-{
-  if (blockedThreads.GetSize() > 0) {
-    PThread * thread = blockedThreads.Dequeue();
-    switch (thread->status) {
-      case PThread::BlockedSem :
-        thread->status = PThread::Waiting;
-        break;
-      case PThread::SuspendedBlockSem :
-        thread->status = PThread::Suspended;
-        break;
-      default:
-        PAssertAlways("Semaphore unblock of thread that is not blocked");
-    }
-    thread->sleepTimer = 0;
-    timeout = PMaxTimeInterval;
-    timeout.Pause();
-  }
-  else if (currentCount < maximumCount)
-    currentCount++;
-}
-
-
-BOOL PSemaphore::WillBlock() const
-{
-  return currentCount == 0;
-}
-
-
-PMutex::PMutex()
-  : PSemaphore(1, 1)
-{
-}
-
-
-PSyncPoint::PSyncPoint()
-  : PSemaphore(0, 1)
-{
-}
-
-
-#endif
-
 /////////////////////////////////////////////////////////////////////////////
 
 void PSyncPointAck::Signal()
