@@ -27,6 +27,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: tlibthrd.cxx,v $
+ * Revision 1.124  2004/02/01 11:23:16  dsandras
+ * Reverted previous Change and removed Yield call from Current (). Fix from Christian Meder <chris@onestepahead.de>. Thanks for your help, Christian!
+ *
+ * Revision 1.123  2004/01/31 13:49:18  dominance
+ * Added 2.6 performance fix as proposed by Christian Meder <chris@onestepahead.de>.
+ *
+ * Revision 1.122  2003/09/17 09:02:15  csoutheren
+ * Removed memory leak detection code
+ *
  * Revision 1.121  2003/05/16 17:40:55  shawn
  * On Mac OS X, thread with the highest priority should use fixed priority
  * scheduling policy.  This avoids starvation caused by desktop activity.
@@ -536,13 +545,7 @@ void PHouseKeepingThread::Main()
 void PProcess::SignalTimerChange()
 {
   if (housekeepingThread == NULL) {
-#if PMEMORY_CHECK
-    BOOL oldIgnoreAllocations = PMemoryHeap::SetIgnoreAllocations(TRUE);
-#endif
     housekeepingThread = new PHouseKeepingThread;
-#if PMEMORY_CHECK
-    PMemoryHeap::SetIgnoreAllocations(oldIgnoreAllocations);
-#endif
   }
 
   BYTE ch;
@@ -1630,9 +1633,6 @@ void PMutex::Signal()
 #endif
 
   PAssertPTHREAD(pthread_mutex_unlock, (&mutex));
-#ifdef P_LINUX
-  PThread::Yield();
-#endif
 }
 
 
